/*
 Copyright (C) 2024 3NSoft Inc.

 This program is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version.

 This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.

 You should have received a copy of the GNU General Public License along with this program. If not, see <http://www.gnu.org/licenses/>.
*/

import { GlobalEventSink, getGlobalEventsSink } from "@/services";
import { useAppStore, useProcessStore } from "@/store";
import { checkForAllUpdates } from "./checkForAllUpdates";
import { updateAppsAndLaunchersInfoInStore } from "./updateAppsAndLaunchersInfoInStore";
import { downloadAndInstallApp } from "./downloadAndInstallApp";
import { ProcessStore } from "@/store/process/types";
import { AppStore } from "@/store/app/types";
import { AppInfo } from "@/types";

/**
 * This "controller" function checks for updates and installs all available.
 * It will restart only idle background processes, while everything else should
 * be restarted by user's action.
 */
export async function checkAndInstallAllUpdates(): Promise<void> {

  const appStore = useAppStore();
  const processStore = useProcessStore();
  const $emit = getGlobalEventsSink();

  await checkForAllUpdates();

  await downloadAndInstallAllAppUpdates(appStore, processStore, $emit);

  await downloadAndInstallPlatformUpdates(appStore, processStore, $emit);

}

async function downloadAndInstallAllAppUpdates(
  appStore: AppStore,
  processStore: ProcessStore,
  $emit: GlobalEventSink
): Promise<void> {

  const appsAndVersionsToInstall = appStore.applicationsInSystem
  .filter(app => !!app.availableUpdates)
  .map(app => ({
    app,
    version: app.availableUpdates![0].version
  }));

  const appsToRestart: AppInfo[] = [];

  for (const { app, version } of appsAndVersionsToInstall) {
    const postInstallState = await downloadAndInstallApp(
      app, version, processStore, $emit
    );
    if (postInstallState
    && (postInstallState !== 'all-done')) {
      appsToRestart.push(app);
    }
  }

  await updateAppsAndLaunchersInfoInStore();

}

async function downloadAndInstallPlatformUpdates(
  appStore: AppStore,
  processStore: ProcessStore,
  $emit: GlobalEventSink
): Promise<void> {

  console.log(`🧐 missing implementation platform download and install`);

}
