/*
 Copyright (C) 2024 3NSoft Inc.

 This program is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version.

 This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.

 You should have received a copy of the GNU General Public License along with this program. If not, see <http://www.gnu.org/licenses/>.
*/

import { NamedProcs } from "@v1nt1248/3nclient-lib/utils";

type WritableFS = web3n.files.WritableFS;
type FileException = web3n.files.FileException;
type DistChannels = web3n.system.apps.DistChannels;

export interface CachedAppDistributionInfo {
  appId: string;
  cacheTS: number;
  channels: DistChannels['channels'];
  mainChannel: DistChannels['main'];
  versions: {
    [channel: string]: string;
  };
}

const UPDATES_CACHE_DIR = '/cached/apps-dist-info';
const CACHE_TTL_MILLIS = 3*60*1000;

const downloader = w3n.system!.apps!.downloader!;


export class AppDistInfoChecker {

  private fs: WritableFS|undefined = undefined;
  private readonly procSync = new NamedProcs();
  private initProc: Promise<void>|undefined = undefined;

  constructor() {
    Object.seal(this);
  }

  init(): Promise<void> {
    this.initProc = w3n.storage!.getAppLocalFS!()
    .then(appFS => appFS.writableSubRoot(UPDATES_CACHE_DIR))
    .then(fs => {
      this.fs = fs;
      this.initProc = undefined;
    });
    return this.initProc;
  }

  private async getCachedAppData(
    appId: string
  ): Promise<CachedAppDistributionInfo|undefined> {
    await this.initProc;
    try {
      return await this.fs!.readJSONFile(appId);
    } catch (exc) {
      if ((exc as FileException).notFound) {
        return;
      } else {
        throw exc;
      }
    }
  }

  private async saveAppData(data: CachedAppDistributionInfo): Promise<void> {
    await this.initProc;
    await this.fs!.writeJSONFile(data.appId, data);
  }

  async getAppDistInfo(
    appId: string, forceInfoDownload = false
  ): Promise<CachedAppDistributionInfo|undefined> {
    const cachedInfo = await this.getCachedAppData(appId);
    if (!forceInfoDownload
    && cachedInfo
    && cacheIsRecent(cachedInfo.cacheTS, Date.now())) {
      return cachedInfo;
    }
    return this.procSync.startOrChain(appId, async () => {
      const data = await downloadAppData(appId);
      await this.saveAppData(data);
      return data;
    }).catch(err => {
      w3n.log('info', `Fail to check app ${appId} updates information`, err);
      return cachedInfo;
    });
  }

}

async function downloadAppData(
  appId: string
): Promise<CachedAppDistributionInfo> {
  const data = {
    appId, versions: {}, cacheTS: Date.now()
  } as CachedAppDistributionInfo;
  const { channels, main } = await downloader.getAppChannels(appId);
  data.channels = channels;
  data.mainChannel = main;
  for (const channel of Object.keys(channels)) {
    try {
      const version = await downloader.getLatestAppVersion(appId, channel);
      data.versions[channel] = version;
    } catch (err) {
      w3n.log('info', `Fail to get latest version of app ${appId} in channel ${channel}`, err);
    }
  }
  return data;
}

function cacheIsRecent(cacheTS: number, now: number): boolean {
  return ((cacheTS + CACHE_TTL_MILLIS) > now);
}
